/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import java.util.Collection;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;

/**
 * This class constructs and manages an editing element for the annotation view. It provides a combo
 * box that displays predefined, fixed values from an annotation value provider.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: AC8E4395DB075EC3FA9BF290373BBD9B
 */
public class ComboBoxEditingSupport extends AnnotationEditingSupportBase {
	/** Combo box cell editor */
	private ComboBoxViewerCellEditor cellEditor = null;

	/** Flag if this {@link ComboBoxEditingSupport} is editable. */
	boolean isEditable;

	/**
	 * Constructor.
	 * 
	 * @param viewer
	 *            Column viewer
	 * @param clazz
	 *            Annotation class handled by this editing support
	 * @param values
	 *            Values for the combo box
	 * @param isEditable
	 *            Flag if combo box should be editable, i.e. if it is possible to create new values
	 */
	public ComboBoxEditingSupport(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, Collection<String> values,
			String instanceKey, boolean isEditable) {
		super(viewer, clazz, instanceKey);
		this.isEditable = isEditable;
		cellEditor = new ComboBoxViewerCellEditor((Composite)getViewer().getControl());
		cellEditor.setLabelProvider(new LabelProvider());
		cellEditor.setContentProvider(new ArrayContentProvider());
		cellEditor.setInput(values);
		cellEditor.getViewer().getCCombo().setEditable(isEditable);
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {
		return cellEditor;
	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(Object element, Object value) {
		if(isEditable && value == null) {
			// New values, i.e values that have been entered into the text input field, and have not
			// been selected from the combo box are indicated by a null value. Hence, fetch the text
			// from the corresponding UI control.
			value = ((CCombo)cellEditor.getControl()).getText();
		}
		super.setValue(element, value);
	}
}
